/**
 * Permalink Manager CSS
 */
#permalink-manager #plugin-name-heading .author-link{color:#959595;font-size:60%;text-decoration:none}
#permalink-manager #permalink-manager-tab-nav .section_upgrade,#permalink-manager #permalink-manager-tab-nav .section_support{background:#555;border-color:#555;color:#fff}
#permalink-manager h4{font-size:18px;margin-bottom:10px}
#permalink-manager h5{font-size:15px;margin-bottom:10px}
#permalink-manager .small{display:block;font-weight:400;color:#888;font-size:80%}
#permalink-manager .mute{color:#888}
#permalink-manager .settings-select{min-width:200px}
#permalink-manager .fixed-table{max-width:100%;table-layout:fixed}
#permalink-manager .margin-top{margin-top:15px}
#permalink-manager .help_tooltip{text-decoration:none;color:#959595}
#permalink-manager .small-select{height:24px;line-height:22px;padding:0 8px 1px;font-size:11px}
#permalink-manager .lead{font-size:125%}
#permalink-manager .licence-info{font-weight:bold}
#permalink-manager p label{font-size:13px;color:#555d66;font-weight:bold}
#permalink-manager a:focus{box-shadow:none}

#permalink-manager .woocommerce-icon:before{font-family:"WooCommerce";font-style:normal}
#permalink-manager .woocommerce-cart:before{content:"\e01d"}
#permalink-manager .woocommerce-logo:before{content:"\e03d"}

#permalink-manager .subsubsub{display:block;width:100%;overflow:hidden;margin-bottom:25px}
#permalink-manager .subsubsub li:before{content:" | "}
#permalink-manager .subsubsub li:first-child:before{content:"";color:#aaa}
#permalink-manager .subsubsub li .dashicons,#permalink-manager .subsubsub li .woocommerce-logo{font-size:100%;line-height:200%;height:auto;width:auto}
#permalink-manager .single-section{clear:both}
#permalink-manager .tablenav{margin:15px 0;height:auto}
#permalink-manager .tablenav.top{margin-top:0}
#permalink-manager .tablenav .actions{padding:0;overflow:visible}
#permalink-manager .tablenav #months-filter{margin-left:10px}
#permalink-manager .search-box{margin:0 0 0 1em}
#permalink-manager th#count a{display:inline-block}
#permalink-manager .post_permalink{margin-top:5px;display:block}
#permalink-manager .post_permalink a,#permalink-manager .post_permalink .dashicons,#permalink-manager .uri_locked .dashicons{font-size:13px;color:#aaa;line-height:1.5em;height:auto;width:auto}
#permalink-manager .post_permalink .dashicons,#permalink-manager .uri_locked .dashicons{font-size:1em}
#permalink-manager .column-count,#permalink-manager .column-post_lang{width:1%}
#permalink-manager .column-count,#permalink-manager .column-post_lang,#permalink-manager th#count{text-align:right}
#permalink-manager .column-item_uri .uri_locked,#permalink-manager .column-item_uri .duplicated_uri_alert{font-weight:bold;margin-top:5px;font-size:80%}
#permalink-manager .extra-info span{white-space:nowrap}

#permalink-manager .info{box-shadow:0 1px 1px 0 rgba(0,0,0,.1);display:block;overflow:hidden;clear:both;border-left:4px solid #dc3232;}
#permalink-manager .alert,#permalink-manager .info{line-height:19px;padding:11px 15px;margin:15px 0}
#permalink-manager .alert p,#permalink-manager .info p{margin-top:0;line-height:200%}
#permalink-manager .alert ul,#permalink-manager .alert ol{margin-top:0;margin-bottom:0}
#permalink-manager .alert p:last-of-type,#permalink-manager .alert li:last-of-type,#permalink-manager .info p:last-of-type{margin-bottom:0}
#permalink-manager .warning{background:#dc3232}
#permalink-manager .warning,#permalink-manager .warning a{color:#fff}
#permalink-manager .info{background:#f9f9f9}
/*#permalink-manager .section-notes,#permalink-manager .short-pre,#permalink-manager .short-textarea{padding:15px;font-size:13px;margin-top:15px;line-height:20px;background:#e5e5e5;border:1px solid #ccc;color:#555}
#permalink-manager .section-notes a,#permalink-manager .section-notes h3{color:#111}
#permalink-manager .section-notes h3,#permalink-manager .section-notes h4{margin-top:0}*/
#permalink-manager .section-notes h4{font-size:14px}
#permalink-manager .section-notes code{font-size:11px}
#permalink-manager .section-notes ol{margin-top:0;margin-bottom:0}
#permalink-manager .section-notes ol li:last-of-type{margin-bottom:0}
#permalink-manager .short-pre,#permalink-manager .short-textarea{height:150px;overflow:auto;max-width:100%;display:block}
#permalink-manager .structure-tags-list{margin-top:15px}
#permalink-manager .duplicated_uri_alert,.permalink-manager .duplicated_uri_alert{display:block;font-weight:bold;font-size:90%;margin-top:8px;color:#dc3232}

#permalink-manager .settings-tabs,#permalink-manager .settings-tabs form,#permalink-manager .settings-tabs form > div{clear:both;overflow:hidden}
#permalink-manager .settings-tabs .subsubsub{margin-bottom:0}
#permalink-manager .settings-tabs form > div{display:none}
#permalink-manager .settings-tabs form > div.active-tab{display:block}

#permalink-manager .checkbox_actions{padding-top:10px;border-top:1px solid #ddd}
#permalink-manager .extra-links,.permalink-manager-screen-options .extra-links{margin-top:5px;font-size:12px}
#permalink-manager .extra-links a,.permalink-manager-screen-options .extra-links a{color:#666;display:inline-block;text-decoration:none}
#permalink-manager .extra-links a:before,.permalink-manager-screen-options .extra-links a:before{content:" | "}
#permalink-manager .extra-links a:first-child:before,.permalink-manager-screen-options .extra-links a:first-child:before{content:""}
.permalink-manager-screen-options{margin-bottom:15px}
#permalink-manager .checkboxes{margin-left:-15px;margin-right:15px;overflow:hidden;display:block}
#permalink-manager .checkboxes > label{display:inline-block;width:33%;float:left;padding:0 15px 5px;box-sizing:border-box;overflow:hidden}
#permalink-manager .checkboxes > label:nth-of-type(3n+1){clear:both}
#permalink-manager .checkboxes > p{font-weight:bold;margin-left:15px;display:block;clear:both;margin-bottom:5px}
#permalink-manager .checkboxes > label + p{padding-top:15px}
#permalink-manager .field-description{font-size:12px}
#permalink-manager .appended-row td{padding-left:0;padding-right:0}
#permalink-manager #tools{margin:0 -15px;padding:0 15px 15px}
#permalink-manager #tools select{min-width:60%}
#permalink-manager fieldset label{margin-bottom:0 !important}

.structure-tags-list{display:block;line-height:175%}
.structure-tags-list pre{white-space:normal;line-height:2em;word-break:keep-all;margin:0}
.structure-tags-list .notice{padding:6px 12px;margin:1em 0;display:block}
.structure-tags-list .permastruct-tag-container{display:flex;flex-direction:row;gap:20px}
.structure-tags-list .permastruct-tag-container > *{flex-basis:400px}
.structure-tags-list .permastruct-tag-container .permastruct-tag-buttons{display:inline-flex;flex-wrap:wrap;gap:10px}
.structure-tags-list .permastruct-tag-container p:first-of-type{margin-top:0;padding-top:0!important}

#permalink-manager .updated-slugs-table .row-title{max-width:400px}

/**
 * Columns
 */
#permalink-manager .columns-container{overflow:hidden;margin-left:-15px;margin-right:-15px;clear:both;display:block}
#permalink-manager .columns-container > *{float:left;padding-left:15px;padding-right:15px;box-sizing:border-box;-webkit-box-sizing:border-box;overflow:hidden;display:block}
#permalink-manager .columns-container span a{color:#aaa}
#permalink-manager .columns-container .column-1_2,#permalink-manager .columns-container .column-2_4{width:50%}
#permalink-manager .columns-container .column-1_3{width:33.33%}
#permalink-manager .columns-container .column-1_3:nth-of-type(3n+1){clear:left}
#permalink-manager .columns-container .column-2_3{width:66.67%}
#permalink-manager .columns-container .column-1_4{width:25%}
#permalink-manager .columns-container .column-3_4{width:75%}

#slugdiv #post_name{width:100%}

/**
 * Edit URI & Duplicate box
 */
.permalink-manager-edit-uri-box{margin:10px -10px}
.permalink-manager-edit-uri-box h2{border-bottom:1px solid #eee}
.permalink-manager-edit-uri-box .inside{padding:0 10px 10px 10px;margin-bottom:0}
.permalink-manager-edit-uri-box .inside > div{font-size:13px;line-height:1.5;margin:1em 0}
.permalink-manager-edit-uri-box .inside > div:last-child,.permalink-manager-edit-uri-box .inside p:last-child{margin-bottom:0}
.permalink-manager-edit-uri-box .strong{display:block;font-weight:bold;margin-bottom:5px}
.permalink-manager-edit-uri-box .toggle-indicator:before{content:"\f142";display:inline-block;font:400 20px/1 dashicons}
.permalink-manager-edit-uri-box .columns-container{margin-left:-10px!important;margin-right:-10px!important}
.permalink-manager-edit-uri-box .columns-container > *{padding-left:10px;padding-right:10px}
.permalink-manager-edit-uri-box .close-button{float:right;width:36px;height:36px;padding:0;cursor:pointer;display:block}
.permalink-manager-edit-uri-box .close-icon{margin-top:4px;-webkit-border-radius:50%;border-radius:50%;text-indent:-1px;color:#72777c;line-height:36px;display:block;text-align:center}
.permalink-manager-edit-uri-box .close-icon:before{content:"\f158";display:inline-block;font:400 20px/1 dashicons;speak:none;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;text-decoration:none!important}
.permalink-manager-edit-uri-box .uri_locked{color:#aaa;margin-top:8px;font-size:80%}
.permalink-manager-edit-uri-box > .uri_locked{font-size:1em;padding:0 10px;color:inherit}
.permalink-manager-edit-uri-box .default-permalink-row,.permalink-manager-edit-uri-box .redirects-row{border-top:1px solid #eee;padding-top:15px}
.permalink-manager-edit-uri-box .default-permalink-row,.permalink-manager-edit-uri-box .default-permastruct-row,.permalink-manager-edit-uri-box .redirects-row{color:#aaa;font-size:90%}

#permalink-manager .restore-default{display:inline-block;border-radius:0;color:#959595;text-decoration:none}
#permalink-manager .restore-default{text-align:right;width:100%}
#permalink-manager .widefat,#permalink-manager #redirect-panel-inside,#permalink-manager #redirect-panel-inside table{width:100%!important;max-width:none;box-sizing:border-box}
#permalink-manager textarea.widefat,#permalink-manager #redirect-panel-inside table{height:auto}
#permalink-manager .heading,#permalink-manager .heading a{margin:0;line-height:1.4;font-weight:600;text-decoration:none;color:#777}
#permalink-manager.permalink-manager-edit-term .permalink-manager-edit-uri-box{display:block!important;margin:0}
#permalink-manager.permalink-manager-edit-term .permalink-manager-edit-uri-box h2,.permalink-manager.postbox .heading{font-size:14px!important;padding:8px 12px;margin:0;line-height:1.4}
#permalink-manager.permalink-manager-edit-term .permalink-manager-edit-uri-box .close-button,#permalink-manager.permalink-manager-edit-term #permalink-manager-toggle{display:none}

#permalink-manager .permalink-manager-duplicate-box table{width:100%;color:#aaa}
#permalink-manager .permalink-manager-duplicate-box table tr td{border-top:1px solid #eee}
#permalink-manager .permalink-manager-duplicate-box table tr:first-child td{border-top:0}
#permalink-manager .permalink-manager-duplicate-box table td{font-size:12px;line-height:1.5em;padding:8px 12px}
#permalink-manager .permalink-manager-duplicate-box table tr td:first-child{width:60%}
#permalink-manager .permalink-manager-duplicate-box table tr td:last-child{text-align:right}
#permalink-manager .permalink-manager-duplicate-box .actions a{text-decoration:none}
#permalink-manager .permalink-manager-duplicate-box .remove-duplicate-link{color:#a00}

.permalink-manager.postbox .heading{border-bottom:3px double #eee}
.permalink-manager.postbox a{color:#444;text-decoration:none;white-space:nowrap}

.edit-post-meta-boxes-area #permalink-manager.postbox .inside{padding:0 24px 24px;background:none}
#permalink-manager .permalink-manager-gutenberg .uri_locked{margin-top:1em}
#permalink-manager .permalink-manager-gutenberg > div{margin-top:15px}
#permalink-manager .permalink-manager-gutenberg .default-permalink-row > *{width:100%;float:none}
#permalink-manager .permalink-manager-gutenberg .restore-default{text-align:left;margin-top:15px}
#permalink-manager .permalink-manager-gutenberg .redirects-row{padding-top:15px}
#permalink-manager .permalink-manager-gutenberg .redirects-panel .heading{background:#eee;padding:15px 10px;margin-bottom:-15px;width:100%;display:block;font-size:110%}
#permalink-manager #redirect-panel-inside, #permalink-manager #redirect-panel-inside p{margin-top:15px}
#permalink-manager .redirects-panel-description *{font-size:11px}

/**
 * Redirect panel
 */
#permalink-manager #redirect-panel-inside{display:none}
#permalink-manager #redirect-panel-inside,#permalink-manager #redirect-panel-inside p{margin-top:15px;height:auto}
#permalink-manager #redirect-panel-inside table{margin-bottom:10px;margin-top:-5px}
#permalink-manager #redirect-panel-inside table td{padding:5px 0;width:100%}
#permalink-manager #redirect-panel-inside table td a{text-decoration:none;text-align:right}
#permalink-manager #redirect-panel-inside .single-section{margin-left:-10px;margin-right:-10px;padding:10px;border-top:1px dashed #eee}
#permalink-manager #redirect-panel-inside .single-section:last-of-type{padding-bottom:0}
#permalink-manager #redirect-panel-inside .single-section:first-of-type{padding-top:0;border:0}
#permalink-manager #redirect-panel-inside .single-section > p:first-of-type{margin-top:0}
.permalink-manager .sample-row{display:none}
.permalink-manager .redirects-panel-description{margin-bottom:10px}
.permalink-manager .redirects-panel-description *{font-size:12px}

/**
 * Coupon panel
 */
#permalink-manager-coupon-url > p:last-of-type{font-size:13px}

/**
 * Tippy.js custom theme
 */
.tippy-tooltip.tippy-pm-theme{text-align:left;font-size:13px}

/**
 * Permastructures
 */
#permalink-manager #permastructs{margin-top:1em}
#permalink-manager .permastruct-row .permastruct-settings{display:none}
#permalink-manager .permastruct-row .permastruct-settings .default-permastruct-row{margin-top:15px}
#permalink-manager .permastruct-row .permastruct-settings .single_checkbox label{font-weight:400}
#permalink-manager .permastruct-row h4{font-size:14px}
#permalink-manager .permastruct-row p{font-size:13px}
#permalink-manager .permastruct-row td label{font-weight:600;margin-top:15px!important;font-size:13px;margin-bottom:5px!important;display:block;clear:both}

#permalink-manager .permastruct-row .permastruct-container{display:flex;flex-direction:row;justify-content:flex-start;gap:15px;flex-wrap:wrap}
#permalink-manager .permastruct-row .permastruct-field-container{display:flex;align-items:center;gap:5px;flex-wrap:wrap;flex-grow:1}
#permalink-manager .permastruct-row .permastruct-field-container span:last-child{flex:1 0 auto;min-width:35%}
#permalink-manager .permastruct-row .permastruct-field-container span:last-child input{width:100%}
#permalink-manager .permastruct-row .permastruct-field:disabled{cursor:not-allowed}
#permalink-manager .permastruct-row .permastruct-buttons{margin:0;font-size:13px;min-width:fit-content;display:flex;gap:10px}
#permalink-manager .permastruct-row .permastruct-buttons > span{height:100%;display:flex}
#permalink-manager .permastruct-row .permastruct-buttons a{border-color:#ddd;padding:0 8px;font-size:inherit;align-content:center}
#permalink-manager .permastruct-row .permastruct-buttons a .dashicons{font-size:initial;line-height:1em;height:1em;vertical-align:middle}

/**
 * TagEditor custom theme & "Stop Words"
 */
#permalink-manager .words-editor{font-size:90%;height:400px;overflow-y:auto;border:1px solid #ddd;-webkit-box-shadow:inset 0 1px 2px rgba(0,0,0,.07);box-shadow:inset 0 1px 2px rgba(0,0,0,.07);background-color:#fff;color:#32373c;outline:0;-webkit-transition:50ms border-color ease-in-out;transition:50ms border-color ease-in-out;}
#permalink-manager .words-editor .tag{background:#eee}
#permalink-manager .words-editor .tag a{color:#000}
#permalink-manager .stop-words-buttons{width:100%}
#permalink-manager .stop-words-buttons tr > td:first-child{padding-left:0;width:50%}
#permalink-manager .stop-words-buttons tr > td:last-child{padding-right:0}

/**
 * Breakpoints
 */
@media only screen and (min-width: 992px) {
	#permalink-manager #permalink-manager-tab-nav .section_debug,#permalink-manager #permalink-manager-tab-nav .section_support{float:right}
    #permalink-manager #permalink-manager-tab-nav .section_upgrade{margin-left:50px}

	#permalink-manager .column-item_uri{width:600px;max-width:100%}
	#permalink-manager .column-item_uri input{width:100%}
}
@media only screen and (max-width: 782px) {
	#permalink-manager .columns-container .column-1_2,#permalink-manager .columns-container .column-1_3,#permalink-manager .columns-container .column-2_3{width:100%}
	#permalink-manager .permalink-manager-duplicate-box table tr td:first-child{width:40%}
}
